@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 14, 2018
+# Last Change: July 21, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -39,7 +39,7 @@ except ImportError:
 # preserve backwards compatibility with older versions of humanfriendly where
 # that function was defined in this module.
 from humanfriendly.compat import HTMLParser, StringIO, coerce_string, name2codepoint, is_unicode, unichr
-from humanfriendly.text import concatenate, format
+from humanfriendly.text import compact_empty_lines, concatenate, format
 from humanfriendly.usage import find_meta_variables, format_usage  # NOQA
 
 ANSI_CSI = '\x1b['
@@ -220,8 +220,7 @@ def ansi_style(**kw):
     and `faint=True` increases the number of available colors to around 24 (it
     may be slightly lower, for example because faint black is just black).
 
-    Support for 8-bit colors
-    ------------------------
+    **Support for 8-bit colors**
 
     In `release 4.7`_ support for 256 color mode was added. While this
     significantly increases the available colors it's not very human friendly
@@ -234,10 +233,9 @@ def ansi_style(**kw):
     mode support would fit into a single screen shot without scrolling :-)
     (I wasn't feeling very creative).
 
-    .. image:: images/ansi-demo.png
+      .. image:: images/ansi-demo.png
 
-    Support for 24-bit colors
-    -------------------------
+    **Support for 24-bit colors**
 
     In `release 4.14`_ support for 24-bit colors was added by accepting a tuple
     or list with three integers representing the RGB (red, green, blue) value
@@ -441,20 +439,20 @@ def connected_to_terminal(stream=None):
         return False
 
 
-def html_to_ansi(data):
+def html_to_ansi(data, callback=None):
     """
     Convert HTML with simple text formatting to text with ANSI escape sequences.
 
     :param data: The HTML to convert (a string).
+    :param callback: Optional callback to pass to :class:`HTMLConverter`.
     :returns: Text with ANSI escape sequences (a string).
 
     Please refer to the documentation of the :class:`HTMLConverter` class for
-    more details about the conversion process (like which tags are supported).
+    details about the conversion process (like which tags are supported) and an
+    example with a screenshot.
     """
-    converter = HTMLConverter()
-    converter.feed(data)
-    converter.close()
-    return converter.output.getvalue()
+    converter = HTMLConverter(callback=callback)
+    return converter(data)
 
 
 def terminal_supports_colors(stream=None):
@@ -665,6 +663,31 @@ class HTMLConverter(HTMLParser):
     - Foreground color: ``<span style="color: #RRGGBB;">``
     - Background color: ``<span style="background-color: #RRGGBB;">``
 
+    Here's a small demonstration:
+
+    .. code-block:: python
+
+       from humanfriendly.text import dedent
+       from humanfriendly.terminal import html_to_ansi
+
+       print(html_to_ansi(dedent('''
+         <b>Hello world!</b>
+         <i>Is this thing on?</i>
+         I guess I can <u>underline</u> or <s>strike-through</s> text?
+         And what about <span style="color: red">color</span>?
+       ''')))
+
+       rainbow_colors = [
+           '#FF0000', '#E2571E', '#FF7F00', '#FFFF00', '#00FF00',
+           '#96BF33', '#0000FF', '#4B0082', '#8B00FF', '#FFFFFF',
+       ]
+       html_rainbow = "".join('<span style="color: %s">o</span>' % c for c in rainbow_colors)
+       print(html_to_ansi("Let's try a rainbow: %s" % html_rainbow))
+
+    Here's what the results look like:
+
+      .. image:: images/html-to-ansi.png
+
     Some more details:
 
     - Nested tags are supported, within reasonable limits.
@@ -675,7 +698,15 @@ class HTMLConverter(HTMLParser):
     - ``<a href="URL">TEXT</a>`` is converted to the format "TEXT (URL)" where
       the uppercase symbols are highlighted in light blue with an underline.
 
-    - ``<br>`` is converted to a plain text line break.
+    - ``<div>``, ``<p>`` and ``<pre>`` tags are considered block level tags
+      and are wrapped in vertical whitespace to prevent their content from
+      "running into" surrounding text. This may cause runs of multiple empty
+      lines to be emitted. As a *workaround* the :func:`__call__()` method
+      will automatically call :func:`.compact_empty_lines()` on the generated
+      output before returning it to the caller. Of course this won't work
+      when `output` is set to something like :data:`sys.stdout`.
+
+    - ``<br>`` is converted to a single plain text line break.
 
     Implementation notes:
 
@@ -690,13 +721,19 @@ class HTMLConverter(HTMLParser):
       :func:`__call__()` method (it makes this use case very easy).
     """
 
+    BLOCK_TAGS = ('div', 'p', 'pre')
+    """The names of tags that are padded with vertical whitespace."""
+
     def __init__(self, *args, **kw):
         """
         Initialize an :class:`HTMLConverter` object.
 
-        :param callback: Optional keyword argument to provide a function that
+        :param callback: Optional keyword argument to specify a function that
                          will be called to process text fragments before they
                          are emitted on the output stream.
+
+                         Note that link text and preformatted text fragments
+                         are not processed by this callback.
         :param output: Optional keyword argument to redirect the output to the
                        given file-like object. If this is not given a new
                        :class:`python3:~io.StringIO` object is created.
@@ -719,7 +756,26 @@ class HTMLConverter(HTMLParser):
         self.feed(data)
         self.close()
         if isinstance(self.output, StringIO):
-            return self.output.getvalue()
+            return compact_empty_lines(self.output.getvalue())
+
+    @property
+    def current_style(self):
+        """Get the current style from the top of the stack (a dictionary)."""
+        return self.stack[-1] if self.stack else {}
+
+    def close(self):
+        """
+        Close previously opened ANSI escape sequences.
+
+        This method overrides the same method in the superclass to ensure that
+        an :data:`.ANSI_RESET` code is emitted when parsing reaches the end of
+        the input but a style is still active. This is intended to prevent
+        malformed HTML from messing up terminal output.
+        """
+        if any(self.stack):
+            self.output.write(ANSI_RESET)
+            self.stack = []
+        HTMLParser.close(self)
 
     def emit_style(self, style=None):
         """
@@ -729,12 +785,12 @@ class HTMLConverter(HTMLParser):
                       :data:`None`, in which case the style at the top of the
                       stack is emitted.
         """
-        if style is None:
-            style = self.stack[-1]
+        # Clear the current text styles.
+        self.output.write(ANSI_RESET)
+        # Apply a new text style?
+        style = self.current_style if style is None else style
         if style:
             self.output.write(ansi_style(**style))
-        else:
-            self.output.write(ANSI_RESET)
 
     def handle_charref(self, value):
         """
@@ -754,9 +810,10 @@ class HTMLConverter(HTMLParser):
             # Link text is captured literally so that we can reliably check
             # whether the text and the URL of the link are the same string.
             self.link_text = data
-        elif self.callback:
-            # Text that is not part of a link is passed to the user defined
-            # callback to allow for arbitrary pre-processing.
+        elif self.callback and self.preformatted_text_level == 0:
+            # Text that is not part of a link and not preformatted text is
+            # passed to the user defined callback to allow for arbitrary
+            # pre-processing.
             data = self.callback(data)
         # All text is emitted unmodified on the output stream.
         self.output.write(data)
@@ -768,8 +825,12 @@ class HTMLConverter(HTMLParser):
         :param tag: The name of the tag (a string).
         """
         if tag in ('a', 'b', 'code', 'del', 'em', 'i', 'ins', 'pre', 's', 'strong', 'span', 'u'):
-            old_style = self.stack.pop(-1)
-            new_style = self.stack[-1]
+            old_style = self.current_style
+            # The following conditional isn't necessary for well formed
+            # HTML but prevents raising exceptions on malformed HTML.
+            if self.stack:
+                self.stack.pop(-1)
+            new_style = self.current_style
             if tag == 'a':
                 if self.urls_match(self.link_text, self.link_url):
                     # Don't render the URL when it's part of the link text.
@@ -783,6 +844,11 @@ class HTMLConverter(HTMLParser):
                     self.output.write(')')
             else:
                 self.emit_style(new_style)
+            if tag in ('code', 'pre'):
+                self.preformatted_text_level -= 1
+        if tag in self.BLOCK_TAGS:
+            # Emit an empty line after block level tags.
+            self.output.write('\n\n')
 
     def handle_entityref(self, name):
         """
@@ -799,6 +865,9 @@ class HTMLConverter(HTMLParser):
         :param tag: The name of the tag (a string).
         :param attrs: A list of tuples with two strings each.
         """
+        if tag in self.BLOCK_TAGS:
+            # Emit an empty line before block level tags.
+            self.output.write('\n\n')
         if tag == 'a':
             self.push_styles(color='blue', bright=True, underline=True)
             # Store the URL that the link points to for later use, so that we
@@ -812,6 +881,7 @@ class HTMLConverter(HTMLParser):
             self.output.write('\n')
         elif tag == 'code' or tag == 'pre':
             self.push_styles(color='yellow')
+            self.preformatted_text_level += 1
         elif tag == 'del' or tag == 's':
             self.push_styles(strike_through=True)
         elif tag == 'em' or tag == 'i':
@@ -895,10 +965,14 @@ class HTMLConverter(HTMLParser):
         1. Make a copy of the current styles (from the top of the stack),
         2. Apply the given `changes` to the copy of the current styles,
         3. Add the new styles to the stack,
-        3. Emit the appropriate ANSI escape sequence to the output stream.
+        4. Emit the appropriate ANSI escape sequence to the output stream.
         """
-        new_style = dict(self.stack[-1])
-        new_style.update(changes)
+        prototype = self.current_style
+        if prototype:
+            new_style = dict(prototype)
+            new_style.update(changes)
+        else:
+            new_style = changes
         self.stack.append(new_style)
         self.emit_style(new_style)
 
@@ -933,11 +1007,12 @@ class HTMLConverter(HTMLParser):
         # Reset our instance variables.
         self.link_text = None
         self.link_url = None
+        self.preformatted_text_level = 0
         if self.output is None or isinstance(self.output, StringIO):
             # If the caller specified something like output=sys.stdout then it
             # doesn't make much sense to negate that choice here in reset().
             self.output = StringIO()
-        self.stack = [dict()]
+        self.stack = []
 
     def urls_match(self, a, b):
         """
