@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 10, 2017
+# Last Change: July 16, 2017
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -55,8 +55,10 @@ __all__ = (
     'TemporaryDirectory',
     'TestCase',
     'configure_logging',
+    'make_dirs',
     'retry',
     'run_cli',
+    'touch',
 )
 
 
@@ -82,6 +84,16 @@ def configure_logging(log_level=logging.DEBUG):
             datefmt='%Y-%m-%d %H:%M:%S')
 
 
+def make_dirs(pathname):
+    """
+    Create missing directories.
+
+    :param pathname: The pathname of a directory (a string).
+    """
+    if not os.path.isdir(pathname):
+        os.makedirs(pathname)
+
+
 def retry(func, timeout=60, exc_type=AssertionError):
     """retry(func, timeout=60, exc_type=AssertionError)
     Retry a function until assertions no longer fail.
@@ -141,6 +153,7 @@ def run_cli(entry_point, *arguments, **options):
               1. The return code (an integer).
               2. The captured output (a string).
     """
+    merged = options.get('merged', False)
     # Add the `program_name' option to the arguments.
     arguments = list(arguments)
     arguments.insert(0, options.pop('program_name', sys.executable))
@@ -178,13 +191,31 @@ def run_cli(entry_point, *arguments, **options):
             returncode = 1
     else:
         logger.debug("Command line entry point returned successfully!")
-    finally:
-        for name, value in (('stdout', stdout), ('stderr', stderr)):
-            if value:
-                logger.debug("Output on %s:\n%s", name, value)
-            else:
-                logger.debug("No output on %s.", name)
-        return returncode, stdout
+    # Always log the output captured on stdout/stderr, to make it easier to
+    # diagnose test failures (but avoid duplicate logging when merged=True).
+    merged_streams = [('merged streams', stdout)]
+    separate_streams = [('stdout', stdout), ('stderr', stderr)]
+    streams = merged_streams if merged else separate_streams
+    for name, value in streams:
+        if value:
+            logger.debug("Output on %s:\n%s", name, value)
+        else:
+            logger.debug("No output on %s.", name)
+    return returncode, stdout
+
+
+def touch(filename):
+    """
+    The equivalent of the UNIX ``touch`` program in Python.
+
+    :param filename: The pathname of the file to touch (a string).
+
+    Note that missing directories are automatically created using
+    :func:`make_dirs()`.
+    """
+    make_dirs(os.path.dirname(filename))
+    with open(filename, 'a'):
+        os.utime(filename, None)
 
 
 class CallableTimedOut(Exception):
