@@ -4,7 +4,7 @@
 # Tests for the `humanfriendly' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: July 14, 2018
+# Last Change: July 21, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """Test suite for the `humanfriendly` package."""
@@ -72,7 +72,7 @@ from humanfriendly.testing import (
     run_cli,
     touch,
 )
-from humanfriendly.text import random_string
+from humanfriendly.text import compact_empty_lines, random_string
 from humanfriendly.usage import (
     find_meta_variables,
     format_usage,
@@ -215,6 +215,21 @@ class HumanFriendlyTestCase(TestCase):
             More {type} template notation
         ''', type='readable') == 'More readable template notation'
 
+    def test_compact_empty_lines(self):
+        """Test :func:`humanfriendly.text.compact_empty_lines()`."""
+        # Simple strings pass through untouched.
+        assert compact_empty_lines('foo') == 'foo'
+        # Horizontal whitespace remains untouched.
+        assert compact_empty_lines('\tfoo') == '\tfoo'
+        # Line breaks should be preserved.
+        assert compact_empty_lines('foo\nbar') == 'foo\nbar'
+        # Vertical whitespace should be preserved.
+        assert compact_empty_lines('foo\n\nbar') == 'foo\n\nbar'
+        # Vertical whitespace should be compressed.
+        assert compact_empty_lines('foo\n\n\nbar') == 'foo\n\nbar'
+        assert compact_empty_lines('foo\n\n\n\nbar') == 'foo\n\nbar'
+        assert compact_empty_lines('foo\n\n\n\n\nbar') == 'foo\n\nbar'
+
     def test_dedent(self):
         """Test :func:`humanfriendly.text.dedent()`."""
         assert dedent('\n line 1\n  line 2\n\n') == 'line 1\n line 2\n'
@@ -836,39 +851,38 @@ class HumanFriendlyTestCase(TestCase):
         assert html_to_ansi("Just some plain text") == "Just some plain text"
         # Hyperlinks.
         assert html_to_ansi('<a href="https://python.org">python.org</a>') == \
-            '\x1b[4;94mpython.org\x1b[0m (\x1b[4;94mhttps://python.org\x1b[0m)'
+            '\x1b[0m\x1b[4;94mpython.org\x1b[0m (\x1b[0m\x1b[4;94mhttps://python.org\x1b[0m)'
         # Make sure `mailto:' prefixes are stripped (they're not at all useful in a terminal).
         assert html_to_ansi('<a href="mailto:peter@peterodding.com">peter@peterodding.com</a>') == \
-            '\x1b[4;94mpeter@peterodding.com\x1b[0m'
+            '\x1b[0m\x1b[4;94mpeter@peterodding.com\x1b[0m'
         # Bold text.
-        assert html_to_ansi("Let's try <b>bold</b>") == \
-            ("Let's try %s" % ansi_wrap("bold", bold=True))
+        assert html_to_ansi("Let's try <b>bold</b>") == "Let's try \x1b[0m\x1b[1mbold\x1b[0m"
         assert html_to_ansi("Let's try <span style=\"font-weight: bold\">bold</span>") == \
-            ("Let's try %s" % ansi_wrap("bold", bold=True))
+            "Let's try \x1b[0m\x1b[1mbold\x1b[0m"
         # Italic text.
         assert html_to_ansi("Let's try <i>italic</i>") == \
-            ("Let's try %s" % ansi_wrap("italic", italic=True))
+            "Let's try \x1b[0m\x1b[3mitalic\x1b[0m"
         assert html_to_ansi("Let's try <span style=\"font-style: italic\">italic</span>") == \
-            ("Let's try %s" % ansi_wrap("italic", italic=True))
+            "Let's try \x1b[0m\x1b[3mitalic\x1b[0m"
         # Underlined text.
         assert html_to_ansi("Let's try <ins>underline</ins>") == \
-            ("Let's try %s" % ansi_wrap("underline", underline=True))
+            "Let's try \x1b[0m\x1b[4munderline\x1b[0m"
         assert html_to_ansi("Let's try <span style=\"text-decoration: underline\">underline</span>") == \
-            ("Let's try %s" % ansi_wrap("underline", underline=True))
+            "Let's try \x1b[0m\x1b[4munderline\x1b[0m"
         # Strike-through text.
         assert html_to_ansi("Let's try <s>strike-through</s>") == \
-            ("Let's try %s" % ansi_wrap("strike-through", strike_through=True))
+            "Let's try \x1b[0m\x1b[9mstrike-through\x1b[0m"
         assert html_to_ansi("Let's try <span style=\"text-decoration: line-through\">strike-through</span>") == \
-            ("Let's try %s" % ansi_wrap("strike-through", strike_through=True))
+            "Let's try \x1b[0m\x1b[9mstrike-through\x1b[0m"
         # Pre-formatted text.
         assert html_to_ansi("Let's try <code>pre-formatted</code>") == \
-            ("Let's try %s" % ansi_wrap("pre-formatted", color='yellow'))
+            "Let's try \x1b[0m\x1b[33mpre-formatted\x1b[0m"
         # Text colors (with a 6 digit hexadecimal color value).
         assert html_to_ansi("Let's try <span style=\"color: #AABBCC\">text colors</s>") == \
-            ("Let's try %s" % ansi_wrap("text colors", color=(0xAA, 0xBB, 0xCC)))
+            "Let's try \x1b[0m\x1b[38;2;170;187;204mtext colors\x1b[0m"
         # Background colors (with an rgb(N, N, N) expression).
         assert html_to_ansi("Let's try <span style=\"background-color: rgb(50, 50, 50)\">background colors</s>") == \
-            ("Let's try %s" % ansi_wrap("background colors", background=(50, 50, 50)))
+            "Let's try \x1b[0m\x1b[48;2;50;50;50mbackground colors\x1b[0m"
         # Line breaks.
         assert html_to_ansi("Let's try some<br>line<br>breaks") == \
             "Let's try some\nline\nbreaks"
@@ -880,6 +894,25 @@ class HumanFriendlyTestCase(TestCase):
         assert html_to_ansi("&lt;") == "<"
         # Check that hexadecimal entities are decoded.
         assert html_to_ansi("&#x26;") == "&"
+        # Check that the text callback is actually called.
+
+        def callback(text):
+            return text.replace(':wink:', ';-)')
+
+        assert ':wink:' not in html_to_ansi('<b>:wink:</b>', callback=callback)
+        # Check that the text callback doesn't process preformatted text.
+        assert ':wink:' in html_to_ansi('<code>:wink:</code>', callback=callback)
+        # Try a somewhat convoluted but nevertheless real life example from my
+        # personal chat archives that causes humanfriendly releases 4.15 and
+        # 4.15.1 to raise an exception.
+        assert html_to_ansi(u'''
+            Tweakers zit er idd nog steeds:<br><br>
+            peter@peter-work&gt; curl -s <a href="tweakers.net">tweakers.net</a> | grep -i hosting<br>
+            &lt;a href="<a href="http://www.true.nl/webhosting/">http://www.true.nl/webhosting/</a>"
+                rel="external" id="true" title="Hosting door True"&gt;&lt;/a&gt;<br>
+            Hosting door &lt;a href="<a href="http://www.true.nl/vps/">http://www.true.nl/vps/</a>"
+                title="VPS hosting" rel="external"&gt;True</a>
+        ''')
 
     def test_generate_output(self):
         """Test the :func:`humanfriendly.terminal.output()` function."""
